@@ -1,6 +1,8 @@
 from __future__ import print_function, division, absolute_import
 
+from functools import partial
 import logging
+import math
 from threading import Thread
 from time import sleep
 
@@ -35,6 +37,8 @@ class LocalCluster(object):
         If False keep the workers in the main calling process
     scheduler_port: int
         Port of the scheduler.  8786 by default, use 0 to choose a random port
+    kwargs: dict
+        Extra worker arguments, will be passed to the Worker constructor.
 
     Examples
     --------
@@ -58,6 +62,7 @@ class LocalCluster(object):
             silence_logs=logging.CRITICAL, diagnostics_port=8787,
             services={'http': HTTPScheduler}, **kwargs):
         self.status = None
+        self.nanny = nanny
         if silence_logs:
             for l in ['distributed.scheduler',
                       'distributed.worker',
@@ -74,7 +79,8 @@ class LocalCluster(object):
         if n_workers is None and threads_per_worker is not None:
             n_workers = max(1, _ncores // threads_per_worker)
         if n_workers and threads_per_worker is None:
-            threads_per_worker = max(1, _ncores // n_workers)
+            # Overcommit threads per worker, rather than undercommit
+            threads_per_worker = max(1, int(math.ceil(_ncores / n_workers)))
 
         self.loop = loop or IOLoop()
         if not self.loop._running:
@@ -92,9 +98,9 @@ class LocalCluster(object):
         if start:
             _start_worker = self.start_worker
         else:
-            _start_worker = lambda *args, **kwargs: self.loop.add_callback(self._start_worker, *args, **kwargs)
+            _start_worker = partial(self.loop.add_callback, self._start_worker)
         for i in range(n_workers):
-            _start_worker(ncores=threads_per_worker, nanny=nanny)
+            _start_worker(ncores=threads_per_worker, nanny=nanny, **kwargs)
         self.status = 'running'
 
         self.diagnostics = None
@@ -111,7 +117,9 @@ class LocalCluster(object):
     __repr__ = __str__
 
     @gen.coroutine
-    def _start_worker(self, port=0, nanny=True, **kwargs):
+    def _start_worker(self, port=0, nanny=None, **kwargs):
+        if nanny is None:
+            nanny = self.nanny
         if nanny:
             W = Nanny
             kwargs['quiet'] = True
@@ -119,7 +127,12 @@ class LocalCluster(object):
             W = Worker
         w = W(self.scheduler.ip, self.scheduler.port, loop=self.loop, **kwargs)
         yield w._start(port)
+
         self.workers.append(w)
+
+        while w.worker_address not in self.scheduler.worker_info:
+            yield gen.sleep(0.01)
+
         raise gen.Return(w)
 
     def start_worker(self, port=0, ncores=0, **kwargs):
@@ -210,6 +223,35 @@ class LocalCluster(object):
                 self.loop.close()
                 del self._thread
 
+    @gen.coroutine
+    def scale_up(self, n, **kwargs):
+        """ Bring the total count of workers up to ``n``
+
+        This function/coroutine should bring the total number of workers up to
+        the number ``n``.
+
+        This can be implemented either as a function or as a Tornado coroutine.
+        """
+        yield [self._start_worker(**kwargs)
+                for i in range(n - len(self.workers))]
+
+    @gen.coroutine
+    def scale_down(self, workers):
+        """ Remove ``workers`` from the cluster
+
+        Given a list of worker addresses this function should remove those
+        workers from the cluster.  This may require tracking which jobs are
+        associated to which worker address.
+
+        This can be implemented either as a function or as a Tornado coroutine.
+        """
+        workers = set(workers)
+        yield [self._stop_worker(w)
+                for w in self.workers
+                if w.worker_address in workers]
+        while workers & set(self.workers):
+            yield gen.sleep(0.01)
+
     def __del__(self):
         self.close()
 
