@@ -21,10 +21,11 @@ import pytest
 
 from distributed import Nanny, Worker
 from distributed.batched import BatchedStream
-from distributed.core import connect, read, write, rpc, dumps
+from distributed.core import connect, read, write, rpc
 from distributed.scheduler import (validate_state, decide_worker,
         Scheduler)
 from distributed.client import _wait
+from distributed.protocol.pickle import dumps
 from distributed.worker import dumps_function, dumps_task
 from distributed.utils_test import (inc, ignoring, dec, gen_cluster, gen_test,
         loop)
@@ -35,6 +36,8 @@ from dask.compatibility import apply
 alice = 'alice:1234'
 bob = 'bob:1234'
 
+stack_duration = defaultdict(lambda: 0)
+
 
 @gen_cluster()
 def test_administration(s, a, b):
@@ -45,21 +48,10 @@ def test_administration(s, a, b):
     assert str(len(s.ncores)) in repr(s)
 
 
-@gen_cluster()
-def test_ready_add_worker(s, a, b):
-    s.add_client(client='client')
-    s.add_worker(address=alice, coerce_address=False)
-
-    s.update_graph(tasks={'x-%d' % i: dumps_task((inc, i)) for i in range(20)},
-                   keys=['x-%d' % i for i in range(20)],
-                   client='client',
-                   dependencies={'x-%d' % i: set() for i in range(20)})
-
-
 @gen_cluster(ncores=[])
 def test_update_state(s):
     s.add_worker(address=alice, ncores=1, coerce_address=False)
-    s.update_graph(tasks={'x': 1, 'y': (inc, 'x')},
+    s.update_graph(tasks={'x': 1, 'y': dumps_task((inc, 'x'))},
                    keys=['y'],
                    dependencies={'y': 'x', 'x': set()},
                    client='client')
@@ -76,13 +68,14 @@ def test_update_state(s):
     assert s.who_wants == {'y': {'client'}}
     assert s.wants_what == {'client': {'y'}}
 
-    s.update_graph(tasks={'a': 1, 'z': (add, 'y', 'a')},
+    s.update_graph(tasks={'a': 1, 'z': dumps_task((add, 'y', 'a'))},
                    keys=['z'],
                    dependencies={'z': {'y', 'a'}},
                    client='client')
 
 
-    assert s.tasks == {'x': 1, 'y': (inc, 'x'), 'a': 1, 'z': (add, 'y', 'a')}
+    assert s.tasks == {'x': 1, 'y': dumps_task((inc, 'x')), 'a': 1,
+                       'z': dumps_task((add, 'y', 'a'))}
     assert s.dependencies == {'x': set(), 'a': set(), 'y': {'x'}, 'z': {'a', 'y'}}
     assert s.dependents == {'z': set(), 'y': {'z'}, 'a': {'z'}, 'x': {'y'}}
 
@@ -98,7 +91,8 @@ def test_update_state(s):
 @gen_cluster(ncores=[])
 def test_update_state_with_processing(s):
     s.add_worker(address=alice, ncores=1, coerce_address=False)
-    s.update_graph(tasks={'x': 1, 'y': (inc, 'x'), 'z': (inc, 'y')},
+    s.update_graph(tasks={'x': 1, 'y': dumps_task((inc, 'x')),
+                          'z': dumps_task((inc, 'y'))},
                    keys=['z'],
                    dependencies={'y': {'x'}, 'x': set(), 'z': {'y'}},
                    client='client')
@@ -118,7 +112,8 @@ def test_update_state_with_processing(s):
 
     assert s.who_has == {'x': {alice}}
 
-    s.update_graph(tasks={'a': (inc, 'x'), 'b': (add,'a','y'), 'c': (inc, 'z')},
+    s.update_graph(tasks={'a': dumps_task((inc, 'x')), 'b': (add,'a','y'),
+                          'c': dumps_task((inc, 'z'))},
                    keys=['b', 'c'],
                    dependencies={'a': {'x'}, 'b': {'a', 'y'}, 'c': {'z'}},
                    client='client')
@@ -153,7 +148,8 @@ def test_update_state_respects_data_in_memory(c, s, a):
 @gen_cluster(ncores=[])
 def test_update_state_supports_recomputing_released_results(s):
     s.add_worker(address=alice, ncores=1, coerce_address=False)
-    s.update_graph(tasks={'x': 1, 'y': (inc, 'x'), 'z': (inc, 'x')},
+    s.update_graph(tasks={'x': 1, 'y': dumps_task((inc, 'x')),
+                          'z': dumps_task((inc, 'x'))},
                    keys=['z'],
                    dependencies={'y': {'x'}, 'x': set(), 'z': {'y'}},
                    client='client')
@@ -178,7 +174,7 @@ def test_update_state_supports_recomputing_released_results(s):
 
     assert s.who_has == {'z': {alice}}
 
-    s.update_graph(tasks={'x': 1, 'y': (inc, 'x')},
+    s.update_graph(tasks={'x': 1, 'y': dumps_task((inc, 'x'))},
                    keys=['y'],
                    dependencies={'y': {'x'}, 'x': set()},
                    client='client')
@@ -199,10 +195,11 @@ def test_decide_worker_with_many_independent_leaves():
     who_has = merge({('x', i * 2): {alice} for i in range(50)},
                     {('x', i * 2 + 1): {bob} for i in range(50)})
     nbytes = {k: 0 for k in who_has}
+    ncores = {alice: 1, bob: 1}
 
     for key in dsk:
-        worker = decide_worker(dependencies, stacks, processing, who_has, {},
-                               {}, set(), nbytes, key)
+        worker = decide_worker(dependencies, stacks, stack_duration, processing,
+                               who_has, {}, {}, set(), nbytes, ncores, key)
         stacks[worker].append(key)
 
     nhits = (len([k for k in stacks[alice] if alice in who_has[('x', k[1])]])
@@ -219,21 +216,22 @@ def test_decide_worker_with_restrictions():
     who_has = {}
     restrictions = {'x': {'alice', 'charlie'}}
     nbytes = {}
-    result = decide_worker(dependencies, stacks, processing, who_has, {},
-                           restrictions, set(), nbytes, 'x')
+    ncores = {alice: 1, bob: 1, charlie: 1}
+    result = decide_worker(dependencies, stacks, stack_duration, processing,
+                          who_has, {}, restrictions, set(), nbytes, ncores, 'x')
     assert result in {alice, charlie}
 
 
     stacks = {alice: [1, 2, 3], bob: [], charlie: [4, 5, 6]}
-    result = decide_worker(dependencies, stacks, processing, who_has, {},
-                           restrictions, set(), nbytes, 'x')
+    result = decide_worker(dependencies, stacks, stack_duration, processing,
+                          who_has, {}, restrictions, set(), nbytes, ncores, 'x')
     assert result in {alice, charlie}
 
     dependencies = {'x': {'y'}}
     who_has = {'y': {bob}}
     nbytes = {'y': 0}
-    result = decide_worker(dependencies, stacks, processing, who_has, {},
-                           restrictions, set(), nbytes, 'x')
+    result = decide_worker(dependencies, stacks, stack_duration, processing,
+                          who_has, {}, restrictions, set(), nbytes, ncores, 'x')
     assert result in {alice, charlie}
 
 
@@ -241,33 +239,36 @@ def test_decide_worker_with_loose_restrictions():
     dependencies = {'x': set()}
     alice, bob, charlie = 'alice:8000', 'bob:8000', 'charlie:8000'
     stacks = {alice: [1, 2, 3], bob: [], charlie: [1]}
+    stack_duration = {alice: 3, bob: 0, charlie: 1}
     processing = {alice: dict(), bob: dict(), charlie: dict()}
     who_has = {}
     nbytes = {}
+    ncores = {alice: 1, bob: 1, charlie: 1}
     restrictions = {'x': {'alice', 'charlie'}}
 
-    result = decide_worker(dependencies, stacks, processing, who_has, {},
-                           restrictions, set(), nbytes, 'x')
+    result = decide_worker(dependencies, stacks, stack_duration, processing,
+                          who_has, {}, restrictions, set(), nbytes, ncores, 'x')
     assert result == charlie
 
-    result = decide_worker(dependencies, stacks, processing, who_has, {},
-                           restrictions, {'x'}, nbytes, 'x')
+    result = decide_worker(dependencies, stacks, stack_duration, processing,
+                          who_has, {}, restrictions, {'x'}, nbytes, ncores, 'x')
     assert result == charlie
 
     restrictions = {'x': {'david', 'ethel'}}
-    result = decide_worker(dependencies, stacks, processing, who_has, {},
-                           restrictions, set(), nbytes, 'x')
+    result = decide_worker(dependencies, stacks, stack_duration, processing,
+                          who_has, {}, restrictions, set(), nbytes, ncores, 'x')
     assert result is None
 
     restrictions = {'x': {'david', 'ethel'}}
-    result = decide_worker(dependencies, stacks, processing, who_has, {},
-                           restrictions, {'x'}, nbytes, 'x')
+    result = decide_worker(dependencies, stacks, stack_duration, processing,
+                          who_has, {}, restrictions, {'x'}, nbytes, ncores, 'x')
+
     assert result == bob
 
 
-
 def test_decide_worker_without_stacks():
-    assert not decide_worker({'x': []}, {}, {}, {}, {}, {}, set(), {}, 'x')
+    assert not decide_worker({'x': []}, {}, {}, {}, {}, {}, {}, set(), {}, {},
+                             'x')
 
 
 def test_validate_state():
@@ -491,9 +492,9 @@ def test_remove_client(s, a, b):
 
 @gen_cluster()
 def test_server_listens_to_other_ops(s, a, b):
-    r = rpc(ip='127.0.0.1', port=s.port)
-    ident = yield r.identity()
-    assert ident['type'] == 'Scheduler'
+    with rpc(ip='127.0.0.1', port=s.port) as r:
+        ident = yield r.identity()
+        assert ident['type'] == 'Scheduler'
 
 
 @gen_cluster()
@@ -530,6 +531,7 @@ def test_add_worker(s, a, b):
 
     assert w.ip in s.host_info
     assert s.host_info[w.ip]['ports'] == set(map(str, [a.port, b.port, w.port]))
+    yield w._close()
 
 
 @gen_cluster()
@@ -800,6 +802,8 @@ def test_ready_add_worker(s, a, b):
                 for w in s.ncores)
     assert len(s.ready) + sum(map(len, s.processing.values())) == 20
 
+    w.scheduler.close_rpc()
+
 
 @gen_cluster()
 def test_broadcast(s, a, b):
@@ -897,7 +901,9 @@ def test_file_descriptors_dont_leak(s):
 @gen_cluster()
 def test_update_graph_culls(s, a, b):
     s.add_client(client='client')
-    s.update_graph(tasks={'x': (inc, 1), 'y': (inc, 'x'), 'z': (inc, 2)},
+    s.update_graph(tasks={'x': dumps_task((inc, 1)),
+                          'y': dumps_task((inc, 'x')),
+                          'z': dumps_task((inc, 2))},
                    keys=['y'],
                    dependencies={'y': 'x', 'x': [], 'z': []},
                    client='client')
@@ -989,14 +995,43 @@ def test_launch_without_blocked_services():
     yield [s.close(), s2.close()]
 
 
-@gen_cluster(ncores=[])
-def test_scatter_no_workers(s):
+@gen_cluster(ncores=[], client=True)
+def test_scatter_no_workers(c, s):
     with pytest.raises(gen.TimeoutError):
         yield gen.with_timeout(timedelta(seconds=0.1),
                               s.scatter(data={'x': dumps(1)}, client='alice'))
 
     w = Worker(s.ip, s.port, ncores=3, ip='127.0.0.1')
-    yield [s.scatter(data={'x': dumps(1)}, client='alice'),
+    yield [c._scatter(data={'x': 1}),
            w._start()]
 
     assert w.data['x'] == 1
+    yield w._close()
+
+
+@gen_cluster(ncores=[])
+def test_scheduler_sees_memory_limits(s):
+    w = Worker(s.ip, s.port, ncores=3, ip='127.0.0.1', memory_limit=12345)
+    yield w._start(0)
+
+    assert s.worker_info[w.address]['memory_limit'] == 12345
+    yield w._close()
+
+
+@gen_cluster(client=True, timeout=1000)
+def test_retire_workers(c, s, a, b):
+    [x] = yield c._scatter([1], workers=a.address)
+    [y] = yield c._scatter([list(range(1000))], workers=b.address)
+
+    assert s.workers_to_close() == [a.address]
+
+    workers = yield s.retire_workers()
+    assert workers == [a.address]
+    assert list(s.ncores) == [b.address]
+
+    assert s.workers_to_close() == []
+
+    assert s.has_what[b.address] == {x.key, y.key}
+
+    workers = yield s.retire_workers()
+    assert not workers
