@@ -8,10 +8,10 @@ from tornado import gen
 
 def test_cluster(loop):
     with cluster() as (s, [a, b]):
-        s = rpc(ip='127.0.0.1', port=s['port'])
-        ident = loop.run_sync(s.identity)
-        assert ident['type'] == 'Scheduler'
-        assert len(ident['workers']) == 2
+        with rpc(ip='127.0.0.1', port=s['port']) as s:
+            ident = loop.run_sync(s.identity)
+            assert ident['type'] == 'Scheduler'
+            assert len(ident['workers']) == 2
 
 
 @gen_cluster(client=True)
